/*
    Copyright (C) 2005-2012  Erik van Pienbroek

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "gui.h"
#include "config_gui.h"

#ifdef HAVE_LIBNOTIFY

#include <libnotify/notify.h>

static void
file_state_changed (NntpgrabGlue *obj, const char *collection_name, const char *subject, const char *real_filename, NGTaskState old_state, NGTaskState new_state, guint64 file_size_remaining, guint64 total_size, guint64 total_size_remaining, gpointer data)
{
    char *msg;
    NotifyNotification *notification;
    ConfigGUIOpts opts;

    opts = config_gui_get_opts();

    switch (new_state) {
        case TASK_STATE_WAITING_FOR_DECODE:
            if (!opts.show_notification_on_file_downloaded) {
                return;
            }

            msg = g_strdup_printf(_("All the parts belonging to the post with subject\n<span style='italic'>%s</span>\nare now downloaded and will soon be decoded"), subject);
#ifdef HAVE_LIBNOTIFY_0_7
            notification = notify_notification_new(_("File download done"), msg, NULL);
#else
            notification = notify_notification_new(_("File download done"), msg, NULL, NULL);
#endif
            g_free(msg);

            break;

        case TASK_STATE_FINISHED_COMPLETE:
        case TASK_STATE_FINISHED_INCOMPLETE:
            if (!opts.show_notification_on_file_decoded) {
                return;
            }

            msg = g_strdup_printf(_("The file belonging to the post with subject\n<span style='italic'>%s</span>\nis now decoded"), subject);
#ifdef HAVE_LIBNOTIFY_0_7
            notification = notify_notification_new(_("File decode done"), msg, NULL);
#else
            notification = notify_notification_new(_("File decode done"), msg, NULL, NULL);
#endif
            g_free(msg);

            break;

        default:
            return;
            break;
    };

    notify_notification_show(notification, NULL);
}

void
notify_initialize(void)
{
    if (!notify_init(PACKAGE)) {
        GtkWidget *windowMain = nntpgrab_gui_base_get_widget("windowMain");
        nntpgrab_gui_base_dialog_show(windowMain, _("Libnotify could not be initialised.\nNotification bubbles will be disabled"), GTK_MESSAGE_ERROR, GTK_BUTTONS_OK);

        return;
    }

    nntpgrab_glue_signal_connect(glue, "file_state_changed", NG_CALLBACK(file_state_changed), NULL);
}

#else /* HAVE_LIBNOTIFY */

void
notify_initialize(void)
{
}

#endif /* HAVE_LIBNOTIFY */
