#include "DialogConnect.h"
#include "ui_DialogConnect.h"

#include <QMessageBox>
#include "ConfigGuiOpts.h"
#include "QNNTPGrabGlue.h"
#include "mainwindow.h"

DialogConnect::DialogConnect(NGApplication *app, QWidget *parent) :
    QDialog(parent),
    ui(new Ui::DialogConnect)
{
    glue = new QNNTPGrabGlue();

    this->app = app;

    ui->setupUi(this);
    ui->btnStartStandalone->setFocus();
    w = NULL;

    if (glue->utilsTestIsServerAlreadyRunning()) {
        ui->btnStartStandalone->setEnabled(false);
    }

    if(!gui_opts.getHostname1().isEmpty()) {
        ui->editHostname->addItem(gui_opts.getHostname1());
    }
    if(!gui_opts.getHostname2().isEmpty()) {
        ui->editHostname->addItem(gui_opts.getHostname2());
    }
    if(!gui_opts.getHostname3().isEmpty()) {
        ui->editHostname->addItem(gui_opts.getHostname3());
    }
    if(!gui_opts.getHostname4().isEmpty()) {
        ui->editHostname->addItem(gui_opts.getHostname4());
    }
    if(!gui_opts.getHostname5().isEmpty()) {
        ui->editHostname->addItem(gui_opts.getHostname5());
    }

}

DialogConnect::~DialogConnect()
{
    if (w) {
        delete w;
    }

    delete ui;
}

void DialogConnect::changeEvent(QEvent *e)
{
    QDialog::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void DialogConnect::doStartStandalone(void)
{
    doConnect(NULL, 0);
}

void DialogConnect::doConnect(QString hostname, int port)
{
    QString errmsg = "";
    QString warnings = "";

    if (!glue->init(hostname, port, NULL, NULL, false, errmsg, warnings)) {
        QMessageBox b(this);

        b.setWindowTitle("NNTPGrab");
        b.setText(QString(tr("Unable to connect to an NNTPGrab server:\n")) + errmsg);
        b.show();
        b.exec();

        return;
    }

    if (!warnings.isEmpty()) {
        qWarning("%s", warnings.toUtf8().data());
    }

    w = new MainWindow(glue, app);
    w->show();

    if (ui->chkDontShowThisDialogAgain->isChecked()) {
        gui_opts.setShowConnectDialog(FALSE);
    }

    this->close();
}

void DialogConnect::on_btnStartStandalone_clicked()
{
    doConnect(NULL, 0);
}

void DialogConnect::on_btnConnect_clicked()
{
    // Have we seen this hostname before ?
    QString hostname = ui->editHostname->currentText();
    if(!hostname.compare(gui_opts.getHostname1())){
        // No need to do anything, item is already at the top of the list
    } else if (!hostname.compare(gui_opts.getHostname2())) {
        // Swap positions
        gui_opts.setHostname2(gui_opts.getHostname1());
        gui_opts.setHostname1(hostname);
    } else if (!hostname.compare(gui_opts.getHostname3())) {
        // Push items 1 and 2 one place down and set the first item to the hostname
        gui_opts.setHostname3(gui_opts.getHostname2());
        gui_opts.setHostname2(gui_opts.getHostname1());
        gui_opts.setHostname1(hostname);
    } else if (!hostname.compare(gui_opts.getHostname4())) {
        gui_opts.setHostname4(gui_opts.getHostname3());
        gui_opts.setHostname3(gui_opts.getHostname2());
        gui_opts.setHostname2(gui_opts.getHostname1());
        gui_opts.setHostname1(hostname);
    } else if (!hostname.compare(gui_opts.getHostname5())) {
        gui_opts.setHostname5(gui_opts.getHostname4());
        gui_opts.setHostname4(gui_opts.getHostname3());
        gui_opts.setHostname3(gui_opts.getHostname2());
        gui_opts.setHostname2(gui_opts.getHostname1());
        gui_opts.setHostname1(hostname);
    } else {
        // Hostname not seen at all, move all the items one place down (forget
        // about the last item) and put the hostname in front
        gui_opts.setHostname5(gui_opts.getHostname4());
        gui_opts.setHostname4(gui_opts.getHostname3());
        gui_opts.setHostname3(gui_opts.getHostname2());
        gui_opts.setHostname2(gui_opts.getHostname1());
        gui_opts.setHostname1(hostname);
    }

    doConnect(hostname, ui->spinPort->value());
}
