#include <time.h>
#include "DownloadQueueItem.h"

DownloadQueueItem::DownloadQueueItem(DownloadQueueItem *parent)
{
    parentItem = parent;
    stateStr = "";
    subject = "";
    poster = "";
    groups = "";
    sizeStr = "";
    sizeRemainingStr = "";
    stamp = QDateTime();
    estimatedTimeRemainingStr = "";
    estimatedTimeToFinishStr = "";
    progress = 0;
    size = 0;
    sizeRemaining = 0;
    realFilename = "";
}

DownloadQueueItem::~DownloadQueueItem()
{
    qDeleteAll(childItems);
}

void DownloadQueueItem::appendChild(DownloadQueueItem *child)
{
    childItems.append(child);
}

void DownloadQueueItem::insertChild(int position, DownloadQueueItem *child)
{
    childItems.insert(position, child);
}

bool DownloadQueueItem::removeChild(DownloadQueueItem *child)
{
    return childItems.removeOne(child);
}

DownloadQueueItem *DownloadQueueItem::child(int row)
{
    return childItems.value(row);
}

int DownloadQueueItem::childCount() const
{
    return childItems.count();
}

int DownloadQueueItem::columnCount() const
{
    return NUM_FIELDS;
}

QVariant DownloadQueueItem::data(int column) const
{
    switch (column) {
        case FIELD_STATE_STR:
            return stateStr;

        case FIELD_SUBJECT:
            return subject;

        case FIELD_POSTER:
            return poster;

        case FIELD_GROUPS:
            return groups;

        case FIELD_FILE_SIZE_STR:
            return sizeStr;

        case FIELD_FILE_SIZE_REMAINING_STR:
            return sizeRemainingStr;

        case FIELD_STAMP:
        {
            char stamp_str[64];
            struct tm *stamp_tm;
            time_t stamp_time_t;
            static int dlqidata_once = 0;

            memset(&stamp_str, 0, sizeof(stamp_str));
            stamp_time_t = stamp.toTime_t();
            if ((stamp_tm = localtime(&stamp_time_t)) == NULL) {
                /* Date could not be parsed */
                return stamp.toString();
            }

            if (strftime(stamp_str, sizeof(stamp_str) - 1, "%c", stamp_tm) == 0) {
                if (!dlqidata_once) {
                    qWarning(__FILE__ ":%i buffer too small", __LINE__);
                    dlqidata_once = 1;
                }
            }

            return QString(stamp_str);
        }

        case FIELD_ESTIMATED_TIME_REMAINING_STR:
            return estimatedTimeRemainingStr;

        case FIELD_ESTIMATED_TIME_TO_FINISH_STR:
            return estimatedTimeToFinishStr;

        case FIELD_PROGRESS:
            return progress;

        case FIELD_FILE_SIZE:
            return size;

        case FIELD_FILE_SIZE_REMAINING:
            return sizeRemaining;

        case FIELD_REAL_FILENAME:
            return realFilename;

        default:
            return QVariant();
    }
}

int DownloadQueueItem::row() const
{
    if (parentItem) {
        return parentItem->childItems.indexOf(const_cast<DownloadQueueItem*>(this));
    }

    return 0;
}

DownloadQueueItem *DownloadQueueItem::parent()
{
    return parentItem;
}

DownloadQueueItem *DownloadQueueItem::lookupSubject(QString subject)
{
    Q_FOREACH(DownloadQueueItem *item, childItems) {
        if (item->subject == subject) {
            return item;
        }
    }

    return NULL;
}

int DownloadQueueItem::rowOfChild(DownloadQueueItem *child) const
{
    return childItems.indexOf(child);
}
