#include "WidgetConfigPlugins.h"
#include "ui_WidgetConfigPlugins.h"

WidgetConfigPlugins::WidgetConfigPlugins(QNNTPGrabGlue *glue, QWidget *parent) :
    QWidget(parent),
    ui(new Ui::WidgetConfigPlugins)
{
    ui->setupUi(this);

    this->glue = glue;

    QList<QString> avail_plugins = glue->getAvailPlugins();
    Q_FOREACH(QString plugin_name, avail_plugins) {
        QNNTPGrabGlue::PluginInfo plugin;

        Q_ASSERT(glue->getPluginInfo(plugin_name, plugin));

        if (plugin_name == "JSON-RPC") {
            if (plugin.is_loaded) {
                ui->groupBoxWebserver->setEnabled(true);
            } else {
                ui->groupBoxWebserver->setEnabled(false);
            }
        } else if (plugin_name == "Logger") {
            if (plugin.is_loaded) {
                ui->groupBoxLogger->setEnabled(true);
            } else {
                ui->groupBoxLogger->setEnabled(false);
            }
        }
    }

    configChanged();

    connect(glue, SIGNAL(configChanged()), SLOT(configChanged()));
    connect(glue, SIGNAL(pluginLoaded(QString,bool)), SLOT(pluginLoaded(QString,bool)));
    connect(glue, SIGNAL(pluginUnloaded(QString)), SLOT(pluginUnloaded(QString)));

    connect(ui->chkEnableWebserver, SIGNAL(toggled(bool)), SLOT(chkEnableWebServer_toggled(bool)));
    connect(ui->spinWebserverPort, SIGNAL(valueChanged(int)), SLOT(spinWebserverPort_value_changed(int)));
    connect(ui->btnWebserverPortApply, SIGNAL(clicked(bool)), SLOT(btnWebserverPortApply_clicked(bool)));
    connect(ui->chkEnableLogger, SIGNAL(toggled(bool)), SLOT(chkEnableLogger_toggled(bool)));
}

WidgetConfigPlugins::~WidgetConfigPlugins()
{
    delete ui;
}

void WidgetConfigPlugins::changeEvent(QEvent *e)
{
    QWidget::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void WidgetConfigPlugins::configChanged()
{
    QNNTPGrabGlue::ConfigOpts opts = glue->configGetOpts();

    if (opts.enable_webserver) {
        ui->chkEnableWebserver->setChecked(true);
    }

    if (opts.enable_logger) {
        ui->chkEnableLogger->setChecked(true);
    }

    ui->spinWebserverPort->setValue(opts.webserver_port);
    ui->btnWebserverPortApply->setEnabled(false);
}

void WidgetConfigPlugins::pluginLoaded(QString plugin_name, bool is_persistent)
{
    Q_UNUSED(is_persistent);

    if (plugin_name == "JSON-RPC") {
        ui->groupBoxWebserver->setEnabled(true);
    } else if (plugin_name == "Logger") {
        ui->groupBoxLogger->setEnabled(true);
    }
}

void WidgetConfigPlugins::pluginUnloaded(QString plugin_name)
{
    if (plugin_name == "JSON-RPC") {
        ui->groupBoxWebserver->setEnabled(false);
    } else if (plugin_name == "Logger") {
        ui->groupBoxLogger->setEnabled(false);
    }
}

void WidgetConfigPlugins::chkEnableWebServer_toggled(bool toggled)
{
    QNNTPGrabGlue::ConfigOpts opts = glue->configGetOpts();

    if (opts.enable_webserver != toggled) {
        opts.enable_webserver = toggled;
        glue->configSetOpts(opts);
    }
}

void WidgetConfigPlugins::spinWebserverPort_value_changed(int value)
{
    Q_UNUSED(value);
    ui->btnWebserverPortApply->setEnabled(true);
}

void WidgetConfigPlugins::btnWebserverPortApply_clicked(bool toggled)
{
    Q_UNUSED(toggled);

    QNNTPGrabGlue::ConfigOpts opts = glue->configGetOpts();

    if (opts.webserver_port != ui->spinWebserverPort->value()) {
        opts.webserver_port = ui->spinWebserverPort->value();
        glue->configSetOpts(opts);
        ui->btnWebserverPortApply->setEnabled(false);
    }
}

void WidgetConfigPlugins::chkEnableLogger_toggled(bool toggled)
{
    QNNTPGrabGlue::ConfigOpts opts = glue->configGetOpts();

    if (opts.enable_logger != toggled) {
        opts.enable_logger = toggled;
        glue->configSetOpts(opts);
    }
}
