#ifndef WIDGETNZBCREATOR_H
#define WIDGETNZBCREATOR_H

#include <QWidget>
#include <QTreeWidgetItem>
#include <QFuture>
#include <QFutureWatcher>
#include <QThreadPool>
#include "QNZBCreator.h"
#include "QNNTPGrabGlue.h"
#include "ProgressBarDelegate.h"
#include "WidgetNZBCreatorDetails.h"

namespace Ui {
    class WidgetNZBCreator;
}

class WidgetNZBCreator : public QWidget {
    Q_OBJECT
public:
    WidgetNZBCreator(QNNTPGrabGlue *glue, QWidget *parent = 0);
    ~WidgetNZBCreator();

protected:
    void changeEvent(QEvent *e);

private:
    Ui::WidgetNZBCreator *ui;
    QNNTPGrabGlue *glue;
    QNZBCreator nzbcreator;
    ProgressBarDelegate *progressbarDelegate;

    /* Retrieve list of newsgroups and maintenance info */
    struct InitResult {
        bool ret;
        bool api_okay;
        QString errmsg;
        QNZBCreator::MaintenanceInfo maintenance_info;
    };

    QFuture<WidgetNZBCreator::InitResult> future_init;
    QFutureWatcher<WidgetNZBCreator::InitResult> watcher_init;

    WidgetNZBCreator::InitResult performInit();

    /* Perform search */
    int results_per_page;
    QNZBCreator::SearchOpts cached_search_opts;

    struct SearchResult {
        bool ret;
        QString errmsg;
        QNZBCreator::SearchResult result;
    };

    QFuture<WidgetNZBCreator::SearchResult> future_search;
    QFutureWatcher<WidgetNZBCreator::SearchResult> watcher_search;

    WidgetNZBCreator::SearchResult performSearch(QNZBCreator::SearchOpts opts);

    /* Retrieve collection details */
    QThreadPool pool_details;
    void performGetDetails(QTreeWidgetItem *item, int collection_id, QDateTime stamp);

    /* Retrieve the NZB */
    struct ImportResult {
        QString collection_name;
        QDateTime oldest_stamp;
        QDateTime newest_stamp;
        QString errmsg;
        QString nzb_data;
    };

    QFuture<WidgetNZBCreator::ImportResult> future_import;
    QFutureWatcher<WidgetNZBCreator::ImportResult> watcher_import;

    WidgetNZBCreator::ImportResult performImport(QString collection_name, QList<int> file_ids, QDateTime oldest_stamp, QDateTime newest_stamp);

private slots:
    void btnSearch_clicked();
    void btnImport_clicked();
    void btnNewerResults_clicked();
    void btnOlderResults_clicked();
    void btnSelectAll_clicked();
    void btnSelectNone_clicked();
    void entryQuery_textChanged(QString text);
    void treeWidget_itemChanged(QTreeWidgetItem *item, int column);
    void treeWidget_expanded(QModelIndex idx);

    void performInit_finished();
    void performSearch_finished();
    void performGetDetails_finished(bool ret, QTreeWidgetItem *item, QList<QNZBCreator::File> files, QString errmsg);
    void performImport_finished();
};

#endif // WIDGETNZBCREATOR_H
