#ifdef HAVE_CONFIG_H
#ifdef _MSC_VER
#include "config.h.win32"
#else
#include "config.h"
#endif
#endif

#include <gmodule.h>
#include <glib-object.h>

#include "php.h"
#include "php_nntpgrab.h"
#include "nntpgrab_glue.h"
#include "nntpgrab_utils.h"

#if ZEND_MODULE_API_NO > 20090626
#define function_entry zend_function_entry
#endif

static function_entry nntpgrab_functions[] = {
    PHP_FE(nntpgrab_connect, NULL)
    PHP_FE(nntpgrab_get_is_connected, NULL)
    PHP_FE(nntpgrab_get_connect_errmsg, NULL)
    PHP_FE(nntpgrab_get_version_info, NULL)
    PHP_FE(nntpgrab_config_get_avail_servers, NULL)
    PHP_FE(nntpgrab_config_get_server_info, NULL)
    PHP_FE(nntpgrab_config_add_server, NULL)
    PHP_FE(nntpgrab_config_del_server, NULL)
    PHP_FE(nntpgrab_config_edit_server, NULL)
    PHP_FE(nntpgrab_config_get_opts, NULL)
    PHP_FE(nntpgrab_config_set_opts, NULL)
    PHP_FE(nntpgrab_config_get_folder_listing, NULL)
    PHP_FE(nntpgrab_schedular_start, NULL)
    PHP_FE(nntpgrab_schedular_stop, NULL)
    PHP_FE(nntpgrab_schedular_get_state, NULL)
    PHP_FE(nntpgrab_schedular_add_task_to_queue, NULL)
    PHP_FE(nntpgrab_schedular_del_task_from_queue, NULL)
    PHP_FE(nntpgrab_schedular_restart_task, NULL)
    PHP_FE(nntpgrab_schedular_save_queue, NULL)
    PHP_FE(nntpgrab_schedular_get_all_tasks, NULL)
    PHP_FE(nntpgrab_schedular_move_task, NULL)
    PHP_FE(nntpgrab_schedular_move_collection, NULL)
    PHP_FE(nntpgrab_debug_get_messages, NULL)
    PHP_FE(nntpgrab_connections_get_speed_history, NULL)
    PHP_FE(nntpgrab_utils_strip_subject, NULL)
    PHP_FE(nntpgrab_utils_calculate_file_size, NULL)
    PHP_FE(nntpgrab_utils_calculate_estimated_time_remaining, NULL)
    PHP_FE(nntpgrab_utils_get_readable_time_remaining, NULL)
    PHP_FE(nntpgrab_utils_get_readable_finished_time, NULL)
    {NULL, NULL, NULL}
};

zend_module_entry nntpgrab_module_entry = {
#if ZEND_MODULE_API_NO >= 20010901
    STANDARD_MODULE_HEADER,
#endif
    PHP_NNTPGRAB_EXTNAME,
    nntpgrab_functions,
    PHP_MINIT(nntpgrab),
    PHP_MSHUTDOWN(nntpgrab),
    NULL,
    NULL,
    NULL,
#if ZEND_MODULE_API_NO >= 20010901
    PHP_NNTPGRAB_VERSION,
#endif
    STANDARD_MODULE_PROPERTIES
};

ZEND_GET_MODULE(nntpgrab)

static NntpgrabGlue *glue = NULL;
static GThread *eventloop_thread = NULL;
static GList *debug_messages = NULL;
GStaticMutex debug_mutex = G_STATIC_MUTEX_INIT;
GStaticMutex traffic_monitor_mutex = G_STATIC_MUTEX_INIT;
static int bytes_received[10] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
static guint64 bytes_received_stamp = 0;
static double bytes_received_average = 0.0;
static char *connection_errmsg = NULL;

static void
on_traffic_monitor_update(NntpgrabGlue *obj, int bytes_received1, int bytes_received2, int bytes_received3, int bytes_received4, int bytes_received5, int bytes_received6, int bytes_received7, int bytes_received8, int bytes_received9, int bytes_received10, guint64 stamp, double average, gpointer data)
{
    g_static_mutex_lock(&traffic_monitor_mutex);

    bytes_received[0] = bytes_received1;
    bytes_received[1] = bytes_received2;
    bytes_received[2] = bytes_received3;
    bytes_received[3] = bytes_received4;
    bytes_received[4] = bytes_received5;
    bytes_received[5] = bytes_received6;
    bytes_received[6] = bytes_received7;
    bytes_received[7] = bytes_received8;
    bytes_received[8] = bytes_received9;
    bytes_received[9] = bytes_received10;
    bytes_received_stamp = stamp;
    bytes_received_average = average;

    g_static_mutex_unlock(&traffic_monitor_mutex);
}

static void
on_log_message(NntpgrabGlue *obj, const char *component, NGLogLevel log_level, const char *msg, gpointer data)
{
    g_static_mutex_lock(&debug_mutex);

    debug_messages = g_list_append(debug_messages, g_strdup(msg));
    if (g_list_length(debug_messages) > 100) {
        g_free(debug_messages->data);
        debug_messages = g_list_remove(debug_messages, debug_messages->data);
    }

    g_static_mutex_unlock(&debug_mutex);
}

static gpointer
eventloop_thread_func(gpointer data)
{
    ng_event_handler_loop_run();

    return NULL;
}

PHP_MINIT_FUNCTION(nntpgrab)
{
    char *errors = NULL;

    g_assert(glue == NULL);

    glue = nntpgrab_glue_new();
    if (!nntpgrab_glue_init(glue, NNTPGRAB_GLUE_VERSION, &errors)) {
        char *msg = g_strdup_printf("NNTPGrab could not be initialised: %s", errors);
        php_error_docref(NULL TSRMLS_CC, E_ERROR, "%s", msg);
        g_free(msg);
        g_free(errors);
        return FAILURE;
    }

    nntpgrab_glue_signal_connect(glue, "log_message", NG_CALLBACK(on_log_message), NULL);
    nntpgrab_glue_signal_connect(glue, "traffic_monitor_update", NG_CALLBACK(on_traffic_monitor_update), NULL);

    return SUCCESS;
}

PHP_MSHUTDOWN_FUNCTION(nntpgrab)
{
    GList *list;

    g_assert(glue != NULL);

    if (eventloop_thread) {
        ng_event_handler_loop_quit();
        g_thread_join(eventloop_thread);
    }

    nntpgrab_glue_destroy(glue);
    glue = NULL;

    g_static_mutex_lock(&debug_mutex);

    list = debug_messages;
    while (list) {
        g_free(list->data);
        list = g_list_next(list);
    }

    g_list_free(debug_messages);
    debug_messages = NULL;

    g_static_mutex_unlock(&debug_mutex);

    if (connection_errmsg) {
        g_free(connection_errmsg);
        connection_errmsg = NULL;
    }

    return SUCCESS;
}

#define ENSURE_IS_CONNECTED                                                                             \
    if (!glue) {                                                                                        \
        php_error_docref(NULL TSRMLS_CC, E_ERROR, "NNTPGrab Core isn't initialised");                   \
        RETURN_NULL();                                                                                  \
    }                                                                                                   \
                                                                                                        \
    if (!nntpgrab_glue_get_is_connected(glue)) {                                                        \
        php_error_docref(NULL TSRMLS_CC, E_ERROR, "There is no connection with the NNTPGrab Server");   \
        RETURN_NULL();                                                                                  \
    }

PHP_FUNCTION(nntpgrab_connect)
{
    char *hostname;
    int hostname_len;
    long port;
    char *err = NULL;
    char *warnings = NULL;
    char *username;
    int username_len;
    char *password;
    int password_len;
    zend_bool use_ssl;

    if (connection_errmsg) {
        g_free(connection_errmsg);
        connection_errmsg = NULL;
    }

    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "slssb", &hostname, &hostname_len, &port, &username, &username_len, &password, &password_len, &use_ssl) == FAILURE) {
        RETURN_NULL();
    }

    if (!nntpgrab_glue_connect(glue, hostname, port, username, password, use_ssl, &err, &warnings)) {
        if (err) {
            char *ret;
            connection_errmsg = err;
            ret = estrdup(err);
            RETURN_STRING(ret, 0);
        } else {
            connection_errmsg = g_strdup("Unknown connection error occured");
            RETURN_FALSE;
        }
    }

    if (warnings) {           // TODO: Proper handling of warnings
        g_free(warnings);
    }

    // Create a mainloop to catch signals coming from the NNTPGrab Server
    if (!eventloop_thread) {
        eventloop_thread = g_thread_create(eventloop_thread_func, NULL, TRUE, NULL);
    }

    RETURN_TRUE;
}

PHP_FUNCTION(nntpgrab_get_is_connected)
{
    if (nntpgrab_glue_get_is_connected(glue)) {
        RETURN_TRUE;
    } else {
        RETURN_FALSE;
    }
}

PHP_FUNCTION(nntpgrab_get_connect_errmsg)
{
    if (connection_errmsg) {
        RETURN_STRING(connection_errmsg, 1);
    } else {
        RETURN_STRING("No errors occured", 1);
    }
}

PHP_FUNCTION(nntpgrab_get_version_info)
{
    array_init(return_value);
    add_assoc_long(return_value, "glue_version", NNTPGRAB_GLUE_VERSION);
    add_assoc_long(return_value, "major_version", MAJOR_VERSION);
    add_assoc_long(return_value, "minor_version", MINOR_VERSION);
    add_assoc_long(return_value, "micro_version", MICRO_VERSION);
    add_assoc_string(return_value, "version", VERSION, 1);
}

PHP_FUNCTION(nntpgrab_config_get_avail_servers)
{
    NGList *list, *servers;

    ENSURE_IS_CONNECTED;

    array_init(return_value);
    servers = nntpgrab_glue_config_get_avail_servers(glue);
    list = servers;
    while (list) {
        add_next_index_string(return_value, list->data, 1);
        list = ng_list_next(list);
    }
    nntpgrab_glue_config_free_avail_servers(glue, servers);
}

PHP_FUNCTION(nntpgrab_config_get_server_info)
{
    char *servername;
    int servername_len;
    NGConfigServer config;

    ENSURE_IS_CONNECTED;

    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &servername, &servername_len) == FAILURE) {
        RETURN_NULL();
    }

    if (!nntpgrab_glue_config_get_server_info(glue, servername, &config)) {
        RETURN_FALSE;
    }

    array_init(return_value);
    add_assoc_string(return_value, "servername", config.servername, 1);
    add_assoc_string(return_value, "hostname", config.hostname, 1);
    add_assoc_long(return_value, "port", (long) config.port);
    add_assoc_string(return_value, "username", config.username, 1);
    add_assoc_string(return_value, "password", config.password, 1);
    add_assoc_long(return_value, "max_threads", config.max_threads);
    add_assoc_long(return_value, "priority", config.priority);
    add_assoc_bool(return_value, "use_ssl", config.use_ssl);
    add_assoc_bool(return_value, "enabled", config.enabled);
}

#define INVALID_DATA_TYPE(field, given_type, expected_type)                         \
    {                                                                               \
        char *msg;                                                                  \
        const char *type;                                                           \
                                                                                    \
        switch(given_type) {                                                        \
            case IS_LONG:                                                           \
                type = "long";                                                      \
                break;                                                              \
                                                                                    \
            case IS_DOUBLE:                                                         \
                type = "double";                                                    \
                break;                                                              \
                                                                                    \
            case IS_BOOL:                                                           \
                type = "bool";                                                      \
                break;                                                              \
                                                                                    \
            case IS_ARRAY:                                                          \
                type = "array";                                                     \
                break;                                                              \
                                                                                    \
            case IS_OBJECT:                                                         \
                type = "object";                                                    \
                break;                                                              \
                                                                                    \
            case IS_STRING:                                                         \
                type = "string";                                                    \
                break;                                                              \
                                                                                    \
            case IS_RESOURCE:                                                       \
                type = "resource";                                                  \
                break;                                                              \
                                                                                    \
            case IS_CONSTANT:                                                       \
                type = "constant";                                                  \
                break;                                                              \
                                                                                    \
            case IS_CONSTANT_ARRAY:                                                 \
                type = "constant_array";                                            \
                break;                                                              \
                                                                                    \
            default:                                                                \
            case IS_NULL:                                                           \
                type = "null";                                                      \
                break;                                                              \
                                                                                    \
        };                                                                          \
                                                                                    \
        msg = g_strdup_printf("The parameter for the field '%s' is of the "         \
                                    "wrong type: given=%s, expected=%s",            \
                                    field, type, expected_type);                    \
        php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s", msg);                     \
        g_free(msg);                                                                \
        RETURN_NULL();                                                              \
    }

#define FIELD_MISSING(field)                                                                \
    {                                                                                       \
        char *msg = g_strdup_printf("The field '%s' was expected but not found", field);    \
        php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s", msg);                             \
        g_free(msg);                                                                        \
        RETURN_NULL();                                                                      \
    }

PHP_FUNCTION(nntpgrab_config_add_server)
{
    zval *arr, **data;
    HashTable *arr_hash;
    HashPosition pointer;
    NGConfigServer server;
    gboolean servername_seen = FALSE;
    gboolean hostname_seen = FALSE;
    gboolean port_seen = FALSE;
    gboolean max_threads_seen = FALSE;
    gboolean priority_seen = FALSE;
    gboolean use_ssl_seen = FALSE;
    gboolean enabled_seen = FALSE;
    char *errmsg = NULL;

    memset(&server, 0, sizeof(NGConfigServer));

    ENSURE_IS_CONNECTED;

    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "a", &arr) == FAILURE) {
        RETURN_NULL();
    }

    arr_hash = Z_ARRVAL_P(arr);

    for(zend_hash_internal_pointer_reset_ex(arr_hash, &pointer); zend_hash_get_current_data_ex(arr_hash, (void**) &data, &pointer) == SUCCESS; zend_hash_move_forward_ex(arr_hash, &pointer)) {

        char *key;
        unsigned int key_len;
        unsigned long index;

        if (zend_hash_get_current_key_ex(arr_hash, &key, &key_len, &index, 0, &pointer) == HASH_KEY_IS_STRING) {
            if (!strncmp(key, "servername", key_len)) {
                if (Z_TYPE_PP(data) != IS_STRING) {
                    INVALID_DATA_TYPE("servername", Z_TYPE_PP(data), "string");
                }

                servername_seen = TRUE;
                strncpy(server.servername, Z_STRVAL_PP(data), sizeof(server.servername));
                continue;
            }

            if (!strncmp(key, "hostname", key_len)) {
                if (Z_TYPE_PP(data) != IS_STRING) {
                    INVALID_DATA_TYPE("hostname", Z_TYPE_PP(data), "string");
                }

                hostname_seen = TRUE;
                strncpy(server.hostname, Z_STRVAL_PP(data), sizeof(server.hostname));
                continue;
            }

            if (!strncmp(key, "port", key_len)) {
                if (Z_TYPE_PP(data) != IS_LONG) {
                    INVALID_DATA_TYPE("port", Z_TYPE_PP(data), "long");
                }

                port_seen = TRUE;
                server.port = Z_LVAL_PP(data);
                continue;
            }

            if (!strncmp(key, "username", key_len)) {
                if (Z_TYPE_PP(data) != IS_STRING) {
                    INVALID_DATA_TYPE("username", Z_TYPE_PP(data), "string");
                }

                strncpy(server.username, Z_STRVAL_PP(data), sizeof(server.username));
                continue;
            }

            if (!strncmp(key, "password", key_len)) {
                if (Z_TYPE_PP(data) != IS_STRING) {
                    INVALID_DATA_TYPE("password", Z_TYPE_PP(data), "string");
                }

                strncpy(server.password, Z_STRVAL_PP(data), sizeof(server.password));
                continue;
            }

            if (!strncmp(key, "max_threads", key_len)) {
                if (Z_TYPE_PP(data) != IS_LONG) {
                    INVALID_DATA_TYPE("max_threads", Z_TYPE_PP(data), "long");
                }

                max_threads_seen = TRUE;
                server.max_threads = Z_LVAL_PP(data);
                continue;
            }

            if (!strncmp(key, "priority", key_len)) {
                if (Z_TYPE_PP(data) != IS_LONG) {
                    INVALID_DATA_TYPE("priority", Z_TYPE_PP(data), "long");
                }

                priority_seen = TRUE;
                server.priority = Z_LVAL_PP(data);
                continue;
            }

            if (!strncmp(key, "use_ssl", key_len)) {
                if (Z_TYPE_PP(data) != IS_BOOL) {
                    INVALID_DATA_TYPE("use_ssl", Z_TYPE_PP(data), "bool");
                }

                use_ssl_seen = TRUE;
                server.use_ssl = Z_BVAL_PP(data);
                continue;
            }

            if (!strncmp(key, "enabled", key_len)) {
                if (Z_TYPE_PP(data) != IS_BOOL) {
                    INVALID_DATA_TYPE("enabled", Z_TYPE_PP(data), "bool");
                }

                enabled_seen = TRUE;
                server.enabled = Z_BVAL_PP(data);
                continue;
            }
        }
    }

    // Do we have all the required arguments ?
    if (!servername_seen) {
        FIELD_MISSING("servername");
    } else if (!hostname_seen) {
        FIELD_MISSING("hostname");
    } else if (!port_seen) {
        FIELD_MISSING("port");
    } else if (!max_threads_seen) {
        FIELD_MISSING("max_threads");
    } else if (!priority_seen) {
        FIELD_MISSING("priority");
    } else if (!use_ssl_seen) {
        FIELD_MISSING("use_ssl");
    } else if (!enabled_seen) {
        FIELD_MISSING("enabled");
    }

    // All ready, execute the request
    if (nntpgrab_glue_config_add_server(glue, server, &errmsg)) {
        RETURN_TRUE;
    } else {
        char *ret = estrdup(errmsg);
        g_free(errmsg);
        RETURN_STRING(ret, 0);
    }
}

PHP_FUNCTION(nntpgrab_config_del_server)
{
    char *servername;
    int servername_len;
    char *errmsg = NULL;

    ENSURE_IS_CONNECTED;

    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &servername, &servername_len) == FAILURE) {
        RETURN_NULL();
    }

    if (nntpgrab_glue_config_del_server(glue, servername, &errmsg)) {
        RETURN_TRUE;
    } else {
        char *ret = estrdup(errmsg);
        g_free(errmsg);
        RETURN_STRING(ret, 0);
    }
}

PHP_FUNCTION(nntpgrab_config_edit_server)
{
    zval *arr, **data;
    HashTable *arr_hash;
    HashPosition pointer;
    NGConfigServer server;
    char *servername_old;
    int servername_old_len;
    gboolean servername_seen = FALSE;
    gboolean hostname_seen = FALSE;
    gboolean port_seen = FALSE;
    gboolean max_threads_seen = FALSE;
    gboolean priority_seen = FALSE;
    gboolean use_ssl_seen = FALSE;
    gboolean enabled_seen = FALSE;
    char *errmsg = NULL;

    memset(&server, 0, sizeof(NGConfigServer));

    ENSURE_IS_CONNECTED;

    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "sa", &servername_old, &servername_old_len, &arr) == FAILURE) {
        RETURN_NULL();
    }

    arr_hash = Z_ARRVAL_P(arr);

    for(zend_hash_internal_pointer_reset_ex(arr_hash, &pointer); zend_hash_get_current_data_ex(arr_hash, (void**) &data, &pointer) == SUCCESS; zend_hash_move_forward_ex(arr_hash, &pointer)) {

        char *key;
        unsigned int key_len;
        unsigned long index;

        if (zend_hash_get_current_key_ex(arr_hash, &key, &key_len, &index, 0, &pointer) == HASH_KEY_IS_STRING) {
            if (!strncmp(key, "servername", key_len)) {
                if (Z_TYPE_PP(data) != IS_STRING) {
                    INVALID_DATA_TYPE("servername", Z_TYPE_PP(data), "string");
                }

                servername_seen = TRUE;
                strncpy(server.servername, Z_STRVAL_PP(data), sizeof(server.servername));
                continue;
            }

            if (!strncmp(key, "hostname", key_len)) {
                if (Z_TYPE_PP(data) != IS_STRING) {
                    INVALID_DATA_TYPE("hostname", Z_TYPE_PP(data), "string");
                }

                hostname_seen = TRUE;
                strncpy(server.hostname, Z_STRVAL_PP(data), sizeof(server.hostname));
                continue;
            }

            if (!strncmp(key, "port", key_len)) {
                if (Z_TYPE_PP(data) != IS_LONG) {
                    INVALID_DATA_TYPE("port", Z_TYPE_PP(data), "long");
                }

                port_seen = TRUE;
                server.port = Z_LVAL_PP(data);
                continue;
            }

            if (!strncmp(key, "username", key_len)) {
                if (Z_TYPE_PP(data) != IS_STRING) {
                    INVALID_DATA_TYPE("username", Z_TYPE_PP(data), "string");
                }

                strncpy(server.username, Z_STRVAL_PP(data), sizeof(server.username));
                continue;
            }

            if (!strncmp(key, "password", key_len)) {
                if (Z_TYPE_PP(data) != IS_STRING) {
                    INVALID_DATA_TYPE("password", Z_TYPE_PP(data), "string");
                }

                strncpy(server.password, Z_STRVAL_PP(data), sizeof(server.password));
                continue;
            }

            if (!strncmp(key, "max_threads", key_len)) {
                if (Z_TYPE_PP(data) != IS_LONG) {
                    INVALID_DATA_TYPE("max_threads", Z_TYPE_PP(data), "long");
                }

                max_threads_seen = TRUE;
                server.max_threads = Z_LVAL_PP(data);
                continue;
            }

            if (!strncmp(key, "priority", key_len)) {
                if (Z_TYPE_PP(data) != IS_LONG) {
                    INVALID_DATA_TYPE("priority", Z_TYPE_PP(data), "long");
                }

                priority_seen = TRUE;
                server.priority = Z_LVAL_PP(data);
                continue;
            }

            if (!strncmp(key, "use_ssl", key_len)) {
                if (Z_TYPE_PP(data) != IS_BOOL) {
                    INVALID_DATA_TYPE("use_ssl", Z_TYPE_PP(data), "bool");
                }

                use_ssl_seen = TRUE;
                server.use_ssl = Z_BVAL_PP(data);
                continue;
            }

            if (!strncmp(key, "enabled", key_len)) {
                if (Z_TYPE_PP(data) != IS_BOOL) {
                    INVALID_DATA_TYPE("enabled", Z_TYPE_PP(data), "bool");
                }

                enabled_seen = TRUE;
                server.enabled = Z_BVAL_PP(data);
                continue;
            }
        }
    }

    // Do we have all the required arguments ?
    if (!servername_seen) {
        FIELD_MISSING("servername");
    } else if (!hostname_seen) {
        FIELD_MISSING("hostname");
    } else if (!port_seen) {
        FIELD_MISSING("port");
    } else if (!max_threads_seen) {
        FIELD_MISSING("max_threads");
    } else if (!priority_seen) {
        FIELD_MISSING("priority");
    } else if (!use_ssl_seen) {
        FIELD_MISSING("use_ssl");
    } else if (!enabled_seen) {
        FIELD_MISSING("enabled");
    }

    // All ready, execute the request
    if (nntpgrab_glue_config_edit_server(glue, servername_old, server, &errmsg)) {
        RETURN_TRUE;
    } else {
        char *ret = estrdup(errmsg);
        g_free(errmsg);
        RETURN_STRING(ret, 0);
    }
}

PHP_FUNCTION(nntpgrab_config_get_opts)
{
    NGConfigOpts opts;

    ENSURE_IS_CONNECTED;

    opts = nntpgrab_glue_config_get_opts(glue);

    array_init(return_value);
    add_assoc_string(return_value, "download_directory", opts.download_directory, 1);
    add_assoc_string(return_value, "temp_directory", opts.temp_directory, 1);
    add_assoc_bool(return_value, "enable_intelligent_par2_downloading", opts.enable_intelligent_par2_downloading);
    add_assoc_bool(return_value, "enable_auto_import", opts.enable_auto_import);
    add_assoc_string(return_value, "auto_import_directory", opts.auto_import_directory, 1);
    add_assoc_bool(return_value, "move_file_after_auto_import", opts.move_file_after_auto_import);
}

PHP_FUNCTION(nntpgrab_config_set_opts)
{
    zval *arr, **data;
    HashTable *arr_hash;
    HashPosition pointer;
    NGConfigOpts opts;
    gboolean download_directory_seen = FALSE;
    gboolean temp_directory_seen = FALSE;
    gboolean enable_intelligent_par2_downloading_seen = FALSE;
    gboolean enable_auto_import_seen = FALSE;
    gboolean auto_import_directory_seen = FALSE;
    gboolean move_file_after_auto_import_seen = FALSE;

    memset(&opts, 0, sizeof(NGConfigOpts));

    ENSURE_IS_CONNECTED;

    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "a", &arr) == FAILURE) {
        RETURN_NULL();
    }

    arr_hash = Z_ARRVAL_P(arr);

    for(zend_hash_internal_pointer_reset_ex(arr_hash, &pointer); zend_hash_get_current_data_ex(arr_hash, (void**) &data, &pointer) == SUCCESS; zend_hash_move_forward_ex(arr_hash, &pointer)) {

        char *key;
        unsigned int key_len;
        unsigned long index;

        if (zend_hash_get_current_key_ex(arr_hash, &key, &key_len, &index, 0, &pointer) == HASH_KEY_IS_STRING) {
            if (!strncmp(key, "download_directory", key_len)) {
                if (Z_TYPE_PP(data) != IS_STRING) {
                    INVALID_DATA_TYPE("download_directory", Z_TYPE_PP(data), "string");
                }

                download_directory_seen = TRUE;
                strncpy(opts.download_directory, Z_STRVAL_PP(data), sizeof(opts.download_directory) - 1);
                continue;
            }

            if (!strncmp(key, "temp_directory", key_len)) {
                if (Z_TYPE_PP(data) != IS_STRING) {
                    INVALID_DATA_TYPE("temp_directory", Z_TYPE_PP(data), "string");
                }

                temp_directory_seen = TRUE;
                strncpy(opts.temp_directory, Z_STRVAL_PP(data), sizeof(opts.temp_directory) - 1);
                continue;
            }

            if (!strncmp(key, "enable_intelligent_par2_downloading", key_len)) {
                if (Z_TYPE_PP(data) != IS_BOOL) {
                    INVALID_DATA_TYPE("enable_intelligent_par2_downloading", Z_TYPE_PP(data), "bool");
                }

                enable_intelligent_par2_downloading_seen = TRUE;
                opts.enable_intelligent_par2_downloading = Z_BVAL_PP(data);
                continue;
            }

            if (!strncmp(key, "enable_auto_import", key_len)) {
                if (Z_TYPE_PP(data) != IS_BOOL) {
                    INVALID_DATA_TYPE("enable_auto_import", Z_TYPE_PP(data), "bool");
                }

                enable_auto_import_seen = TRUE;
                opts.enable_auto_import = Z_BVAL_PP(data);
                continue;
            }

            if (!strncmp(key, "auto_import_directory", key_len)) {
                if (Z_TYPE_PP(data) != IS_STRING) {
                    INVALID_DATA_TYPE("auto_import_directory", Z_TYPE_PP(data), "string");
                }

                auto_import_directory_seen = TRUE;
                strncpy(opts.auto_import_directory, Z_STRVAL_PP(data), sizeof(opts.auto_import_directory) - 1);
                continue;
            }

            if (!strncmp(key, "move_file_after_auto_import", key_len)) {
                if (Z_TYPE_PP(data) != IS_BOOL) {
                    INVALID_DATA_TYPE("move_file_after_auto_import", Z_TYPE_PP(data), "bool");
                }

                move_file_after_auto_import_seen = TRUE;
                opts.move_file_after_auto_import = Z_BVAL_PP(data);
                continue;
            }
        }
    }

    // Do we have all the required arguments ?
    if (!download_directory_seen) {
        FIELD_MISSING("download_directory");
    } else if (!temp_directory_seen) {
        FIELD_MISSING("temp_directory");
    } else if (!enable_intelligent_par2_downloading_seen) {
        FIELD_MISSING("enable_intelligent_par2_downloading");
    } else if (!enable_auto_import_seen) {
        FIELD_MISSING("enable_auto_import");
    } else if (!auto_import_directory_seen) {
        FIELD_MISSING("auto_import_directory");
    } else if (!move_file_after_auto_import_seen) {
        FIELD_MISSING("move_file_after_auto_import");
    }

    // All ready, execute the request
    nntpgrab_glue_config_set_opts(glue, opts);

    RETURN_TRUE;
}

PHP_FUNCTION(nntpgrab_config_get_folder_listing)
{
    char *parent;
    int parent_len;
    NGList *folders = NULL;

    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &parent, &parent_len) == FAILURE) {
        RETURN_NULL();
    }

    ENSURE_IS_CONNECTED;

    if (!nntpgrab_glue_config_get_folder_listing(glue, parent, &folders)) {
        RETURN_FALSE;
    } else {
        NGList *list = folders;

        array_init(return_value);

        while (list) {
            NNTPGrabFolder *folder = list->data;

            //add_next_index_string(return_value, dirname, 1);
            add_assoc_bool(return_value, folder->folder, folder->has_subfolders);
            list = ng_list_next(list);
        }

        nntpgrab_glue_config_free_folder_listing(glue, folders);
    }
}

PHP_FUNCTION(nntpgrab_schedular_start)
{
    ENSURE_IS_CONNECTED;

    if (nntpgrab_glue_schedular_start(glue)) {
        RETURN_TRUE;
    } else {
        RETURN_FALSE;
    }
}

PHP_FUNCTION(nntpgrab_schedular_stop)
{
    zend_bool wait;

    ENSURE_IS_CONNECTED;

    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "b", &wait) == FAILURE) {
        RETURN_NULL();
    }

    if (nntpgrab_glue_schedular_stop(glue, (gboolean) wait)) {
        RETURN_TRUE;
    } else {
        RETURN_FALSE;
    }
}

PHP_FUNCTION(nntpgrab_schedular_get_state)
{
    ENSURE_IS_CONNECTED;

    RETURN_LONG(nntpgrab_glue_schedular_get_state(glue));
}

PHP_FUNCTION(nntpgrab_schedular_add_task_to_queue)
{
    char *collection_name;
    int collection_name_len;
    char *subject;
    int subject_len;
    char *poster;
    int poster_len;
    long  stamp;
    long  file_size;
    NGList *list;
    NGList *groups_list;
    zval *groups;
    NGList *parts_list;
    zval *parts;
    char *errmsg = NULL;
    zval **data;
    HashTable *arr_hash;
    HashPosition pointer;
    HashTable *arr_hash2;
    HashPosition pointer2;
    zval **data2;

    ENSURE_IS_CONNECTED;

    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "sssllaa", &collection_name, &collection_name_len, &subject, &subject_len,
                                                                    &poster, &poster_len, &stamp, &file_size, &groups, &parts) == FAILURE) {
        RETURN_NULL();
    }

    // Convert the array with groups to a GList*
    arr_hash = Z_ARRVAL_P(groups);
    groups_list = NULL;

    for(zend_hash_internal_pointer_reset_ex(arr_hash, &pointer); zend_hash_get_current_data_ex(arr_hash, (void**) &data, &pointer) == SUCCESS; zend_hash_move_forward_ex(arr_hash, &pointer)) {

        if (Z_TYPE_PP(data) != IS_STRING) {
            INVALID_DATA_TYPE("groups", Z_TYPE_PP(data), "string");
        }

        groups_list = ng_list_append(groups_list, Z_STRVAL_PP(data));
    }

    // Convert the array with parts to a GList*
    arr_hash = Z_ARRVAL_P(parts);
    parts_list = NULL;

    for(zend_hash_internal_pointer_reset_ex(arr_hash, &pointer); zend_hash_get_current_data_ex(arr_hash, (void**) &data, &pointer) == SUCCESS; zend_hash_move_forward_ex(arr_hash, &pointer)) {

        NNTPGrabPart *part = g_malloc0(sizeof(NNTPGrabPart));

        arr_hash2 = Z_ARRVAL_PP(data);
        for (zend_hash_internal_pointer_reset_ex(arr_hash2, &pointer2); zend_hash_get_current_data_ex(arr_hash2, (void**) &data2, &pointer2) == SUCCESS; zend_hash_move_forward_ex(arr_hash2, &pointer2)) {

            char *key;
            unsigned int key_len;
            unsigned long index;

            if (zend_hash_get_current_key_ex(arr_hash2, &key, &key_len, &index, 0, &pointer2) == HASH_KEY_IS_STRING) {
                if (!strncmp(key, "message_id", key_len)) {
                    if (Z_TYPE_PP(data2) != IS_STRING) {
                        INVALID_DATA_TYPE("message_id", Z_TYPE_PP(data2), "string");
                    }

                    strncpy(part->message_id, Z_STRVAL_PP(data2), sizeof(part->message_id));
                    continue;
                }

                if (!strncmp(key, "partnum", key_len)) {
                    if (Z_TYPE_PP(data2) != IS_LONG) {
                        INVALID_DATA_TYPE("partnum", Z_TYPE_PP(data2), "long");
                    }

                    part->part_num = Z_LVAL_PP(data2);
                    continue;
                }

                if (!strncmp(key, "size", key_len)) {
                    if (Z_TYPE_PP(data2) != IS_LONG) {
                        INVALID_DATA_TYPE("size", Z_TYPE_PP(data2), "long");
                    }

                    part->size = Z_LVAL_PP(data2);
                    continue;
                }
            }
        }

        if (strlen(part->message_id) > 0) {
            parts_list = ng_list_append(parts_list, part);
        }
    }

    // Try to add the task
    if (!nntpgrab_glue_schedular_add_task_to_queue(glue, collection_name, subject, poster, stamp, file_size, groups_list, parts_list, &errmsg)) {
        if (errmsg) {
            // FIXME: This should be handled by a call-by-reference'd PHP function
            php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s", errmsg);
            g_free(errmsg);
        }

        list = parts_list;
        while (list) {
            g_free(list->data);
            list = ng_list_next(list);
        }

        ng_list_free(groups_list);
        ng_list_free(parts_list);
        RETURN_FALSE;
    } else {
        list = parts_list;
        while (list) {
            g_free(list->data);
            list = ng_list_next(list);
        }

        ng_list_free(groups_list);
        ng_list_free(parts_list);
        RETURN_TRUE;
    }
}

PHP_FUNCTION(nntpgrab_schedular_del_task_from_queue)
{
    char *collection_name;
    int collection_name_len;
    char *subject;
    int subject_len;
    char *errmsg = NULL;

    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ss", &collection_name, &collection_name_len, &subject, &subject_len) == FAILURE) {
        RETURN_NULL();
    }

    if (!nntpgrab_glue_schedular_del_task_from_queue(glue, collection_name, subject, &errmsg)) {
        if (errmsg) {
            // FIXME: This should be handled by a call-by-reference'd PHP function
            php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s", errmsg);
            g_free(errmsg);
        }
        RETURN_FALSE;
    } else {
        RETURN_TRUE;
    }
}

PHP_FUNCTION(nntpgrab_schedular_restart_task)
{
    char *collection_name;
    int collection_name_len;
    char *subject;
    int subject_len;
    char *errmsg = NULL;

    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ss", &collection_name, &collection_name_len, &subject, &subject_len) == FAILURE) {
        RETURN_NULL();
    }

    if (!nntpgrab_glue_schedular_restart_task(glue, collection_name, subject, &errmsg)) {
        if (errmsg) {
            // FIXME: This should be handled by a call-by-reference'd PHP function
            php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s", errmsg);
            g_free(errmsg);
        }
        RETURN_FALSE;
    } else {
        RETURN_TRUE;
    }
}

PHP_FUNCTION(nntpgrab_schedular_save_queue)
{
    char *errmsg = NULL;

    if (!nntpgrab_glue_schedular_save_queue(glue, &errmsg)) {
        if (errmsg) {
            // FIXME: This should be handled by a call-by-reference'd PHP function
            php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s", errmsg);
            g_free(errmsg);
        }

        RETURN_FALSE;
    } else {
        RETURN_TRUE;
    }
}

struct _foreach_task_data
{
    zval *return_value;
    zval *collection;
    zval *files;
    zval *file;
};

static void
foreach_collection_func (const char *collection_name, const char *poster, guint64 total_size, guint64 total_size_remaining, int position, gpointer data)
{
    struct _foreach_task_data *task_data = data;
    long l_total_size;
    long l_total_size_remaining;

    total_size /= 1024;
    total_size_remaining /= 1024;

    l_total_size = total_size;
    l_total_size_remaining = total_size_remaining;

    task_data->collection = NULL;
    MAKE_STD_ZVAL(task_data->collection);
    array_init(task_data->collection);
    add_next_index_zval(task_data->return_value, task_data->collection);

    add_assoc_string(task_data->collection, "collection_name", (char*) collection_name, 1);
    add_assoc_string(task_data->collection, "poster", (char*) poster, 1);
    add_assoc_long(task_data->collection, "total_size", l_total_size);                      // The total size needs to be divided by 1024 as PHP doesn't know 64bit long integers, so we keep the size in KB here
    add_assoc_long(task_data->collection, "total_size_remaining", l_total_size_remaining);

    task_data->files = NULL;
    MAKE_STD_ZVAL(task_data->files);
    array_init(task_data->files);
    add_assoc_zval(task_data->collection, "files", task_data->files);
}

static void
foreach_file_func (const char *collection_name, const char *subject, const char *poster, gint64 stamp, guint64 file_size, guint64 file_size_remaining, int position, int num_parts_total, int num_parts_downloaded, int num_parts_failed, NGTaskState status, const char *filename, gpointer data)
{
    struct _foreach_task_data *task_data = data;

    task_data->file = NULL;
    MAKE_STD_ZVAL(task_data->file);
    array_init(task_data->file);
    add_index_zval(task_data->files, position, task_data->file);

    add_assoc_string(task_data->file, "subject", (char*) subject, 1);
    add_assoc_string(task_data->file, "poster", (char*) poster, 1);
    add_assoc_long(task_data->file, "stamp", (long) stamp);
    add_assoc_long(task_data->file, "file_size", (long) file_size / 1024);
    add_assoc_long(task_data->file, "file_size_remaining", (long) file_size_remaining / 1024);
    add_assoc_long(task_data->file, "num_parts_total", num_parts_total);
    add_assoc_long(task_data->file, "num_parts_downloaded", num_parts_downloaded);
    add_assoc_long(task_data->file, "num_parts_failed", num_parts_failed);
    add_assoc_long(task_data->file, "status", status);
    add_assoc_string(task_data->file, "filename", (char*) filename, 1);
}

static void
foreach_group_func (const char *collection_name, const char *subject, const char *group, gpointer data)
{
    struct _foreach_task_data *task_data = data;
    zval *groups = NULL;

    MAKE_STD_ZVAL(groups);
    array_init(groups);
    add_assoc_zval(task_data->file, "groups", groups);

    add_next_index_string(groups, (char*) group, 1);
}

PHP_FUNCTION(nntpgrab_schedular_get_all_tasks)
{
    struct _foreach_task_data task_data;

    ENSURE_IS_CONNECTED;

    array_init(return_value);
    task_data.return_value = return_value;

    nntpgrab_glue_schedular_foreach_task(glue, foreach_collection_func, foreach_file_func, foreach_group_func, &task_data);
}

PHP_FUNCTION(nntpgrab_schedular_move_task)
{
    char *collection_name_src;
    int collection_name_src_len;
    char *collection_name_dest;
    int collection_name_dest_len;
    char *subject;
    int subject_len;
    int position_dest;

    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "sssl", &collection_name_src, &collection_name_src_len, &subject, &subject_len, &collection_name_dest, &collection_name_dest_len, &position_dest) == FAILURE) {
        RETURN_NULL();
    }

    if (!nntpgrab_glue_schedular_move_task(glue, collection_name_src, subject, collection_name_dest, position_dest)) {
        RETURN_FALSE;
    } else {
        RETURN_TRUE;
    }
}

PHP_FUNCTION(nntpgrab_schedular_move_collection)
{
    char *collection_name;
    int collection_name_len;
    int new_position;

    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "sl", &collection_name, &collection_name_len, &new_position) == FAILURE) {
        RETURN_NULL();
    }

    if (!nntpgrab_glue_schedular_move_collection(glue, collection_name, new_position)) {
        RETURN_FALSE;
    } else {
        RETURN_TRUE;
    }
}

PHP_FUNCTION(nntpgrab_debug_get_messages)
{
    GList *list;

    array_init(return_value);
    g_static_mutex_lock(&debug_mutex);

    list = debug_messages;
    while (list) {
        char *msg = list->data;

        add_next_index_string(return_value, msg, 1);

        list = g_list_next(list);
    }

    g_static_mutex_unlock(&debug_mutex);
}

PHP_FUNCTION(nntpgrab_connections_get_speed_history)
{
    zval *history = NULL;
    int i;

    g_static_mutex_lock(&traffic_monitor_mutex);

    array_init(return_value);

    MAKE_STD_ZVAL(history);
    array_init(history);
    add_assoc_zval(return_value, "history", history);

    for (i = 0; i < 10; i++) {
        add_next_index_long(history, bytes_received[i]);
    }

    add_assoc_long(return_value, "timestamp", (long) bytes_received_stamp); // Does this work okay on 64bit ?
    add_assoc_double(return_value, "average", bytes_received_average);

    g_static_mutex_unlock(&traffic_monitor_mutex);
}

PHP_FUNCTION(nntpgrab_utils_strip_subject)
{
    char *subject;
    int subject_len;
    char *subject_without_partnum = NULL;
    int file_num = -1;
    int total_files = -1;
    char *filename = NULL;
    char *extension = NULL;
    NNTPFileType file_type = NNTP_FILE_TYPE_UNKNOWN;
    int par2_startnum = -1;
    int num_par2_blocks = -1;
    int part_num = -1;
    int total_parts = -1;

    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &subject, &subject_len) == FAILURE) {
        RETURN_NULL();
    }

    if (!nntpgrab_utils_strip_subject(subject, &subject_without_partnum, &file_num, &total_files, &filename, &extension, &file_type, &par2_startnum, &num_par2_blocks, &part_num, &total_parts)) {
        RETURN_FALSE;
    }

    array_init(return_value);
    add_assoc_string(return_value, "subject", subject, 1);

    if (subject_without_partnum) {
        add_assoc_string(return_value, "subject_without_partnum", subject_without_partnum, 1);
        g_free(subject_without_partnum);
    }

    if (file_num > -1) {
        add_assoc_long(return_value, "file_num", file_num);
    }

    if (total_files > -1) {
        add_assoc_long(return_value, "total_files", total_files);
    }

    if (filename) {
        add_assoc_string(return_value, "filename", filename, 1);
        g_free(filename);
    }

    if (extension) {
        add_assoc_string(return_value, "extension", extension, 1);
        g_free(extension);
    }

    add_assoc_long(return_value, "file_type", file_type);

    if (par2_startnum > -1) {
        add_assoc_long(return_value, "par2_startnum", par2_startnum);
    }

    if (num_par2_blocks > -1) {
        add_assoc_long(return_value, "num_par2_blocks", num_par2_blocks);
    }

    if (part_num > -1) {
        add_assoc_long(return_value, "part_num", part_num);
    }

    if (total_parts > -1) {
        add_assoc_long(return_value, "total_parts", total_parts);
    }
}

PHP_FUNCTION(nntpgrab_utils_calculate_file_size)
{
    char ret[64];
    int file_size;

    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &file_size) == FAILURE) {
        RETURN_NULL();
    }

    memset(&ret, 0, sizeof(ret));
    nntpgrab_utils_calculate_file_size(((guint64) file_size) * 1024, ret, sizeof(ret) - 1);     // All sizes in the PHP module are in kbytes

    RETURN_STRING(ret, 1);
}

PHP_FUNCTION(nntpgrab_utils_calculate_estimated_time_remaining)
{
    zval *arr, **data;
    HashTable *arr_hash;
    HashPosition pointer;
    int file_size;
    int bytes_received1 = -1;
    int bytes_received2 = -1;
    int bytes_received3 = -1;
    int bytes_received4 = -1;
    int bytes_received5 = -1;
    int bytes_received6 = -1;
    int bytes_received7 = -1;
    int bytes_received8 = -1;
    int bytes_received9 = -1;
    int bytes_received10 = -1;

    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "al", &arr, &file_size) == FAILURE) {
        RETURN_NULL();
    }

    arr_hash = Z_ARRVAL_P(arr);

    for(zend_hash_internal_pointer_reset_ex(arr_hash, &pointer); zend_hash_get_current_data_ex(arr_hash, (void**) &data, &pointer) == SUCCESS; zend_hash_move_forward_ex(arr_hash, &pointer)) {

        char *key;
        unsigned int key_len;
        unsigned long index;

        if (zend_hash_get_current_key_ex(arr_hash, &key, &key_len, &index, 0, &pointer) == HASH_KEY_IS_LONG) {
            if (index == 0) {
                if (Z_TYPE_PP(data) != IS_LONG) {
                    INVALID_DATA_TYPE("0", Z_TYPE_PP(data), "long");
                }

                bytes_received1 = Z_LVAL_PP(data);
                continue;
            }

            if (index == 1) {
                if (Z_TYPE_PP(data) != IS_LONG) {
                    INVALID_DATA_TYPE("1", Z_TYPE_PP(data), "long");
                }

                bytes_received2 = Z_LVAL_PP(data);
                continue;
            }

            if (index == 2) {
                if (Z_TYPE_PP(data) != IS_LONG) {
                    INVALID_DATA_TYPE("2", Z_TYPE_PP(data), "long");
                }

                bytes_received3 = Z_LVAL_PP(data);
                continue;
            }

            if (index == 3) {
                if (Z_TYPE_PP(data) != IS_LONG) {
                    INVALID_DATA_TYPE("3", Z_TYPE_PP(data), "long");
                }

                bytes_received4 = Z_LVAL_PP(data);
                continue;
            }

            if (index == 4) {
                if (Z_TYPE_PP(data) != IS_LONG) {
                    INVALID_DATA_TYPE("4", Z_TYPE_PP(data), "long");
                }

                bytes_received5 = Z_LVAL_PP(data);
                continue;
            }

            if (index == 5) {
                if (Z_TYPE_PP(data) != IS_LONG) {
                    INVALID_DATA_TYPE("5", Z_TYPE_PP(data), "long");
                }

                bytes_received6 = Z_LVAL_PP(data);
                continue;
            }

            if (index == 6) {
                if (Z_TYPE_PP(data) != IS_LONG) {
                    INVALID_DATA_TYPE("6", Z_TYPE_PP(data), "long");
                }

                bytes_received7 = Z_LVAL_PP(data);
                continue;
            }

            if (index == 7) {
                if (Z_TYPE_PP(data) != IS_LONG) {
                    INVALID_DATA_TYPE("7", Z_TYPE_PP(data), "long");
                }

                bytes_received8 = Z_LVAL_PP(data);
                continue;
            }

            if (index == 8) {
                if (Z_TYPE_PP(data) != IS_LONG) {
                    INVALID_DATA_TYPE("8", Z_TYPE_PP(data), "long");
                }

                bytes_received9 = Z_LVAL_PP(data);
                continue;
            }

            if (index == 9) {
                if (Z_TYPE_PP(data) != IS_LONG) {
                    INVALID_DATA_TYPE("9", Z_TYPE_PP(data), "long");
                }

                bytes_received10 = Z_LVAL_PP(data);
                continue;
            }
        }
    }

    // Do we have all the required arguments ?
    if (bytes_received1 == -1) {
        FIELD_MISSING("0");
    } else if (bytes_received2 == -1) {
        FIELD_MISSING("1");
    } else if (bytes_received3 == -1) {
        FIELD_MISSING("2");
    } else if (bytes_received4 == -1) {
        FIELD_MISSING("3");
    } else if (bytes_received5 == -1) {
        FIELD_MISSING("4");
    } else if (bytes_received6 == -1) {
        FIELD_MISSING("5");
    } else if (bytes_received7 == -1) {
        FIELD_MISSING("6");
    } else if (bytes_received8 == -1) {
        FIELD_MISSING("7");
    } else if (bytes_received9 == -1) {
        FIELD_MISSING("8");
    } else if (bytes_received10 == -1) {
        FIELD_MISSING("9");
    }

    RETURN_LONG(nntpgrab_utils_calculate_estimated_time_remaining(  bytes_received1, bytes_received2, bytes_received3,
                                                                    bytes_received4, bytes_received5, bytes_received6,
                                                                    bytes_received7, bytes_received8, bytes_received9,
                                                                    bytes_received10, ((guint64) file_size) * 1024));
}

PHP_FUNCTION(nntpgrab_utils_get_readable_time_remaining)
{
    char ret[64];
    int time_remaining;

    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &time_remaining) == FAILURE) {
        RETURN_NULL();
    }

    memset(&ret, 0, sizeof(ret));
    nntpgrab_utils_get_readable_time_remaining(time_remaining, ret, sizeof(ret) - 1);

    RETURN_STRING(ret, 1);
}

PHP_FUNCTION(nntpgrab_utils_get_readable_finished_time)
{
    char ret[64];
    int time_remaining;

    if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &time_remaining) == FAILURE) {
        RETURN_NULL();
    }

    memset(&ret, 0, sizeof(ret));
    nntpgrab_utils_get_readable_finished_time(time_remaining, ret, sizeof(ret) - 1);

    RETURN_STRING(ret, 1);
}
